<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Permission\Models\Permission;
use EventPortal\Domain\Permission\Models\Role;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Support\Arr;
use Spatie\Snapshots\MatchesSnapshots;

class UserDataShareTest extends TestCase
{
    /**
     * @var Admin
     */
    protected $admin;

    /**
     * Just the base permission 'event.manage'
     * @var Permission
     */
    protected $permission;

    /**
     * @var array
     */
    protected $payload;

    use DatabaseMigrations, MatchesSnapshots;

    /** @test */
    public function it_contains_permissions(): void
    {
        $this->setPayload();
        $userPayload = Arr::get($this->payload, 'user');

        $this->assertEquals($this->admin->user->id, Arr::get($userPayload, 'id'));
        $this->assertEquals($this->admin->allPermissions, Arr::get($userPayload, 'permissions'));
    }

    /** @test */
    public function it_matches_format(): void
    {
        $this->setPayload();
        $this->assertMatchesJsonSnapshot($this->payload);
    }

    /** @test */
    public function it_gets_all_permissions_and_roles(): void
    {
        $roleOne = factory(Role::class)->create(['name' => 'fucc']);
        $permissionsOne = factory(Permission::class, 3)->create();

        $roleOne->syncPermissions($permissionsOne);

        $roleTwo = factory(Role::class)->create(['name' => 'succ']);
        $permissionsTwo = factory(Permission::class, 6)->create();

        $roleTwo->syncPermissions($permissionsTwo);

        $this->admin->assignRole(['succ', 'fucc']);
        $this->setPayload();
        $userPayload = Arr::get($this->payload, 'user');

        $this->assertCount(10, Arr::get($userPayload, 'permissions'));
        $this->assertCount(2, Arr::get($userPayload, 'roles'));
    }

    public function setUp(): void
    {
        parent::setUp();

        $this->admin = factory(Admin::class)->create();
        $this->permission = factory(Permission::class)->create([
            'name' => 'event.index',
        ]);
        $this->admin->givePermissionTo($this->permission);
    }

    protected function setPayload(): void
    {
        $this->payload = $this->actingAs($this->admin->user)->get(route('admin.event.index'))->props();
    }
}
